// file      : xsd/cxx/xml/dom/error-handler.ixx
// author    : Boris Kolpackov <boris@codesynthesis.com>
// copyright : Copyright (c) 2005 Code Synthesis Tools CC
// license   : GNU GPL v2 + exceptions; see accompanying LICENSE file

#ifndef XSD_CXX_XML_DOM_ERROR_HANDLER_IXX
#define XSD_CXX_XML_DOM_ERROR_HANDLER_IXX

#include <iostream>

#include <xsd/cxx/xml/string.hxx>

#include <xercesc/dom/DOMError.hpp>

#endif // XSD_CXX_XML_DOM_ERROR_HANDLER_IXX

#if defined(XSD_USE_CHAR) || !defined(XSD_USE_WCHAR)

#ifndef XSD_CXX_XML_DOM_ERROR_HANDLER_IXX_CHAR
#define XSD_CXX_XML_DOM_ERROR_HANDLER_IXX_CHAR

namespace xsd
{
  namespace cxx
  {
    namespace xml
    {
      namespace dom
      {
        template<>
        inline
        bool error_handler<char>::
        handleError (xercesc::DOMError const& e)
        {
          using namespace xercesc;

          if (e.getSeverity() != DOMError::DOM_SEVERITY_WARNING)
            failed_ = true;

          if (eh_)
            return eh_->handleError (e);
          else
          {
            std::cerr << transcode<char> (e.getLocation()->getURI()) << ':'
                      << e.getLocation()->getLineNumber() << ':'
                      << e.getLocation()->getColumnNumber();

            switch (e.getSeverity())
            {
            case DOMError::DOM_SEVERITY_WARNING:
              {
                std::cerr << " warning: ";
                break;
              }
            default:
              {
                std::cerr << " error: ";
                break;
              }
            }

            std::cerr << transcode<char> (e.getMessage()) << std::endl;

            return true;
          }
        }
      }
    }
  }
}

#endif // XSD_CXX_XML_DOM_ERROR_HANDLER_IXX_CHAR
#endif // XSD_USE_CHAR


#if defined(XSD_USE_WCHAR) || !defined(XSD_USE_CHAR)

#ifndef XSD_CXX_XML_DOM_ERROR_HANDLER_IXX_WCHAR
#define XSD_CXX_XML_DOM_ERROR_HANDLER_IXX_WCHAR

namespace xsd
{
  namespace cxx
  {
    namespace xml
    {
      namespace dom
      {
        template<>
        inline
        bool error_handler<wchar_t>::
        handleError (xercesc::DOMError const& e)
        {
          using namespace xercesc;

          if (e.getSeverity() != DOMError::DOM_SEVERITY_WARNING)
            failed_ = true;

          if (eh_)
            return eh_->handleError (e);
          else
          {
            std::wcerr << transcode<wchar_t> (e.getLocation()->getURI()) << L':'
                       << e.getLocation()->getLineNumber() << L':'
                       << e.getLocation()->getColumnNumber();

            switch (e.getSeverity())
            {
            case DOMError::DOM_SEVERITY_WARNING:
              {
                std::wcerr << L" warning: ";
                break;
              }
            default:
              {
                std::wcerr << L" error: ";
                break;
              }
            }

            std::wcerr << transcode<wchar_t> (e.getMessage()) << std::endl;

            return true;
          }
        }
      }
    }
  }
}

#endif // XSD_CXX_XML_DOM_ERROR_HANDLER_IXX_WCHAR
#endif // XSD_USE_WCHAR
